@@ -1,3 +1,4 @@
+from dataclasses import is_dataclass, replace
 from enum import Enum
 from itertools import chain
 from typing import (
@@ -17,8 +18,6 @@ from typing import (
     Union,
 )
 
-from dataclasses import is_dataclass, replace
-
 from apischema.conversion import Converter, InputVisitorMixin, OutputVisitorMixin
 from apischema.data import to_data
 from apischema.dataclasses import (
@@ -144,6 +143,9 @@ class SchemaBuilder(Visitor[Schema, JSONSchema]):
             check_constraint(schema, constraint_by_type[cls])
         return JSONSchema(type=JSONType.from_type(cls), **_to_dict(schema))
 
+    def subprimitive(self, cls: Type, superclass: Type, schema: Schema) -> JSONSchema:
+        return self.primitive(superclass, _override(schema, cls))
+
     def union(self, alternatives: Sequence[Type], schema: Schema) -> JSONSchema:
         any_of = []
         for cls in alternatives:
