@@ -11,12 +11,12 @@ from ipaddress import (
 )
 from uuid import UUID
 
-from apischema.conversion import inout_model, input_converter, output_converter
+from apischema.conversion import inout_str, input_converter, output_converter
 from apischema.schema import schema
 
 # ==================== uuid ====================
 
-inout_model(str)(UUID)
+inout_str(UUID)
 schema(format="uuid")(UUID)
 
 # ==================== datetime ====================
@@ -36,22 +36,22 @@ if sys.version_info >= (3, 7):
 
 # ==================== ipaddress ====================
 
-inout_model(str)(IPv4Address)
+inout_str(IPv4Address)
 schema(format="ipv4")(IPv4Address)
 
-inout_model(str)(IPv4Interface)
+inout_str(IPv4Interface)
 schema(format="ipv4")(IPv4Interface)
 
-inout_model(str)(IPv4Network)
+inout_str(IPv4Network)
 schema(format="ipv4")(IPv4Network)
 
-inout_model(str)(IPv6Address)
+inout_str(IPv6Address)
 schema(format="ipv6")(IPv6Address)
 
-inout_model(str)(IPv6Interface)
+inout_str(IPv6Interface)
 schema(format="ipv6")(IPv6Interface)
 
-inout_model(str)(IPv6Network)
+inout_str(IPv6Network)
 schema(format="ipv6")(IPv6Network)
 
 # ==================== pattern ====================
