@@ -26,7 +26,6 @@ Number = Union[int, float]
 
 PRIMITIVE_TYPES = {str, int, bool, float, NoneType}
 
-# Hack before PEP 585 ...
 if sys.version_info >= (3, 7):
     ITERABLE_TYPES = {
         collections.abc.Iterable: tuple,
@@ -41,6 +40,8 @@ if sys.version_info >= (3, 7):
         set: set,
     }
 
+    LIST_TYPE = list
+
     MAPPING_TYPES = {
         collections.abc.Mapping: dict,
         collections.abc.MutableMapping: dict,
@@ -59,6 +60,8 @@ else:
         Set: set,
     }
 
+    LIST_TYPE = list
+
     MAPPING_TYPES = {Mapping: dict, MutableMapping: dict, Dict: dict}
 
 
