@@ -1,13 +1,33 @@
+from typing import (
+    Any,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+    overload,
+)
+
 from dataclasses import Field, dataclass, field
-from typing import (Any, Dict, Iterable, List, Mapping, Optional, Sequence,
-                    Tuple, Union, overload)
 
 from apischema.alias import ALIAS_METADATA
 
 ErrorMsg = str
 Error = Union[ErrorMsg, Tuple[Any, ErrorMsg]]
 
-ROOT_KEY = "."
+
+@dataclass
+class LocalizedError:
+    loc: Sequence[str]
+    err: Union[ErrorMsg, Sequence[ErrorMsg]]
+
+    def __post_init__(self):
+        if not isinstance(self.err, str) and len(self.err) == 1:
+            self.err = self.err[0]
 
 
 @dataclass
@@ -15,33 +35,32 @@ class ValidationError(Exception):
     messages: Sequence[ErrorMsg] = field(default_factory=list)
     children: Mapping[str, "ValidationError"] = field(default_factory=dict)
 
-    def flat(self) -> Iterable[Tuple[Tuple[str, ...], Sequence[ErrorMsg]]]:
+    def flat(self) -> Iterator[Tuple[Tuple[str, ...], Sequence[ErrorMsg]]]:
         if self.messages:
             yield (), self.messages
         for child_path, child in self.children.items():
             for path, errors in child.flat():
                 yield (child_path, *path), errors
 
-    @property
-    def flatten(self) -> Mapping[Tuple[str, ...], Sequence[ErrorMsg]]:
-        return dict(self.flat())
+    def format(self) -> Iterator[LocalizedError]:
+        for loc, err in self.flat():
+            yield LocalizedError(loc, err)
 
 
 @overload
-def merge(err1: Optional[ValidationError], err2: ValidationError
-          ) -> ValidationError:
+def merge(err1: Optional[ValidationError], err2: ValidationError) -> ValidationError:
     ...
 
 
 @overload
-def merge(err1: ValidationError, err2: Optional[ValidationError]
-          ) -> ValidationError:
+def merge(err1: ValidationError, err2: Optional[ValidationError]) -> ValidationError:
     ...
 
 
 @overload
-def merge(err1: Optional[ValidationError], err2: Optional[ValidationError]
-          ) -> Optional[ValidationError]:
+def merge(
+    err1: Optional[ValidationError], err2: Optional[ValidationError]
+) -> Optional[ValidationError]:
     ...
 
 
@@ -50,10 +69,13 @@ def merge(err1: Optional[ValidationError], err2: Optional[ValidationError]):
         return err2
     if err2 is None:
         return err1
-    return ValidationError([*err1.messages, *err2.messages], {
-        key: merge(err1.children.get(key), err2.children.get(key))  # type: ignore
-        for key in err1.children.keys() | err2.children.keys()
-    })
+    return ValidationError(
+        [*err1.messages, *err2.messages],
+        {
+            key: merge(err1.children.get(key), err2.children.get(key))  # type: ignore
+            for key in err1.children.keys() | err2.children.keys()
+        },
+    )
 
 
 def exception(err: Exception) -> str:
@@ -64,8 +86,7 @@ def _rec_build_error(path: Sequence[str], msg: ErrorMsg) -> ValidationError:
     if not path:
         return ValidationError([msg])
     else:
-        return ValidationError(
-            children={path[0]: _rec_build_error(path[1:], msg)})
+        return ValidationError(children={path[0]: _rec_build_error(path[1:], msg)})
 
 
 def _check_error_path(path: Any) -> Sequence[str]:
@@ -79,8 +100,9 @@ def _check_error_path(path: Any) -> Sequence[str]:
         elif isinstance(elt, int):
             path[i] = str(elt)
         if not isinstance(path[i], str):
-            raise TypeError(f"Bad error path, expected Field, int or str,"
-                            f" found {type(i)}")
+            raise TypeError(
+                f"Bad error path, expected Field, int or str," f" found {type(i)}"
+            )
     return path
 
 
@@ -96,6 +118,7 @@ def build_from_errors(errors: Iterable[Error]) -> ValidationError:
         if not path:
             messages.append(msg)
         else:
-            children[path[0]] = merge(children.get(path[0]),
-                                      _rec_build_error(path[1:], msg))
+            children[path[0]] = merge(
+                children.get(path[0]), _rec_build_error(path[1:], msg)
+            )
     return ValidationError(messages, children)
