@@ -6,7 +6,7 @@ from dataclasses import is_dataclass
 from apischema.types import (
     ITERABLE_TYPES,
     MAPPING_TYPES,
-    PRIMITIVE_TYPE,
+    PRIMITIVE_TYPES,
 )
 from apischema.typing import (
     Literal,
@@ -18,7 +18,7 @@ from apischema.typing import (
     get_type_hints,
 )
 
-PRIMITIVE_TYPE_IDS = set(map(id, PRIMITIVE_TYPE))
+PRIMITIVE_TYPE_IDS = set(map(id, PRIMITIVE_TYPES))
 
 
 class Unsupported(TypeError):
@@ -46,6 +46,9 @@ class Visitor(Generic[Arg, Return]):
     def primitive(self, cls: Type, arg: Arg) -> Return:
         raise NotImplementedError()
 
+    def subprimitive(self, cls: Type, superclass: Type, arg: Arg) -> Return:
+        raise NotImplementedError()
+
     def union(self, alternatives: Sequence[Type], arg: Arg) -> Return:
         raise NotImplementedError()
 
@@ -151,6 +154,9 @@ class Visitor(Generic[Arg, Return]):
             return self.any(arg)
         if isinstance(cls, _LiteralMeta):
             return self.literal(cls.__values__, arg)
+        for primitive in PRIMITIVE_TYPES:
+            if issubclass(cls, primitive):
+                return self.subprimitive(cls, primitive, arg)
         if isinstance(cls, NamedTupleMeta):
             if hasattr(cls, "__annotations__"):
                 types = get_type_hints(cls, include_extras=True)
