@@ -1,5 +1,4 @@
-from apischema.validation.dependencies import (find_dependencies,
-                                               find_end_dependencies)
+from apischema.validation.dependencies import find_all_dependencies, find_dependencies
 
 
 def test_find_dependencies():
@@ -12,13 +11,15 @@ def test_find_dependencies():
 
 def test_find_end_dependencies():
     class Test:
+        class_var = ""
+
         def __init__(self):
             self.a = 0
             self.b = {}
 
         def pseudo_validate(self):
             if self.a not in self.method(0):
-                yield "error"
+                yield self.class_var
 
         def method(self, arg):
             res = list(self.c)
@@ -29,4 +30,4 @@ def test_find_end_dependencies():
         def c(self):
             return self.b.values()
 
-    assert find_end_dependencies(Test, Test.pseudo_validate) == {"a", "b"}
+    assert find_all_dependencies(Test, Test.pseudo_validate) == {"a", "b", "class_var"}
